# -*- coding: utf-8 -*-
from collective.cover.testing import ALL_CONTENT_TYPES
from collective.cover.tests.base import TestTileMixin
from collective.cover.tiles.carousel import UUIDSFieldDataConverter
from collective.cover.widgets.textlinessortable import TextLinesSortableWidget
from covertile.cycle2.testing import INTEGRATION_TESTING
from covertile.cycle2.tiles.carousel import CarouselTile
from covertile.cycle2.tiles.carousel import ICarouselTile
from plone.tiles.interfaces import ITileDataManager
from plone.uuid.interfaces import IUUID
from UserDict import UserDict

import unittest


class CarouselTileTestCase(TestTileMixin, unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        super(CarouselTileTestCase, self).setUp()
        self.tile = CarouselTile(self.cover, self.request)
        self.tile.__name__ = u'covertile.cycle2.carousel'
        self.tile.id = u'test'

    def _update_tile_data(self):
        # tile's data attribute is cached; reinstantiate it
        self.tile = self.cover.restrictedTraverse(
            '@@{0}/{1}'.format(str(self.tile.__name__), str(self.tile.id)))

    @unittest.expectedFailure  # FIXME: raises BrokenImplementation
    def test_interface(self):
        self.interface = ICarouselTile
        self.klass = CarouselTile
        super(CarouselTileTestCase, self).test_interface()

    def test_default_configuration(self):
        self.assertTrue(self.tile.is_configurable)
        self.assertTrue(self.tile.is_droppable)
        self.assertTrue(self.tile.is_editable)

    def test_accepted_content_types(self):
        self.assertEqual(self.tile.accepted_ct(), ALL_CONTENT_TYPES)

    def test_tile_is_empty(self):
        self.assertTrue(self.tile.is_empty())

    def test_paused(self):
        # An empty tile is not paused (Autoplay is on)
        self.assertEqual(self.tile.paused, 'false')

        # Add content to the tile - Autoplay still on
        obj = self.portal['my-image']
        self.tile.populate_with_object(obj)
        self.assertIn('data-cycle-paused="false"', self.tile())

        # Turn Autoplay off - test Cycle2 option set
        data_mgr = ITileDataManager(self.tile)
        data = data_mgr.get()
        data['autoplay'] = False
        data_mgr.set(data)
        self._update_tile_data()
        self.assertEqual(self.tile.paused, 'true')
        self.assertIn('data-cycle-paused="true"', self.tile())

    def test_crud(self):
        # we start with an empty tile
        self.assertTrue(self.tile.is_empty())

        # now we add a couple of objects to the list
        obj1 = self.portal['my-document']
        obj2 = self.portal['my-image']

        self.tile.populate_with_object(obj1)
        self.tile.populate_with_object(obj2)

        self._update_tile_data()

        # Document should not have been added
        self.assertEqual(len(self.tile.results()), 1)
        self.assertNotIn(obj1, self.tile.results())
        self.assertIn(obj2, self.tile.results())

        # next, we replace the list of objects with a different one
        obj3 = self.portal['my-image1']
        self.tile.replace_with_objects([IUUID(obj3, None)])
        self._update_tile_data()
        self.assertNotIn(obj1, self.tile.results())
        self.assertNotIn(obj2, self.tile.results())
        self.assertIn(obj3, self.tile.results())

        # finally, we remove it from the list; the tile must be empty again
        self.tile.remove_item(obj3.UID())
        self._update_tile_data()
        self.assertTrue(self.tile.is_empty())

    def test_internal_structure(self):
        # we start with an empty tile
        self.assertTrue(self.tile.is_empty())
        uuids = ITileDataManager(self.tile).get().get('uuids', None)

        self.assertIsNone(uuids)

        # now we add an image
        obj1 = self.portal['my-image']

        self.tile.populate_with_object(obj1)

        uuids = ITileDataManager(self.tile).get().get('uuids', None)

        self.assertIsInstance(uuids, (dict, UserDict))
        self.assertTrue(len(uuids) == 1)
        self.assertTrue(uuids[obj1.UID()]['order'] == u'0')

        # now we add a second image
        obj2 = self.portal['my-image1']

        self.tile.populate_with_object(obj2)

        uuids = ITileDataManager(self.tile).get().get('uuids', None)

        self.assertIsInstance(uuids, (dict, UserDict))
        self.assertTrue(len(uuids) == 2)
        self.assertTrue(uuids[obj1.UID()]['order'] == u'0')
        self.assertTrue(uuids[obj2.UID()]['order'] == u'1')

    def test_custom_title(self):
        # we start with an empty tile
        self.assertTrue(self.tile.is_empty())
        uuids = ITileDataManager(self.tile).get().get('uuids', None)

        self.assertIsNone(uuids)

        # now we 2 elements
        obj1 = self.portal['my-document']
        obj2 = self.portal['my-image']

        self.tile.populate_with_uuids([
            obj1.UID(), obj2.UID()
        ])

        # For obj2 we will assign a custom_title

        uuids = ITileDataManager(self.tile).get().get('uuids', None)
        uuids[obj2.UID()]['custom_title'] = u'New Title'

        title1 = self.tile.get_title(obj1)
        title2 = self.tile.get_title(obj2)

        # Document object should be the same as Title
        self.assertEqual(title1, obj1.Title())
        # First image should return the custom Title
        self.assertEqual(title2, u'New Title')

    def test_custom_description(self):
        # we start with an empty tile
        self.assertTrue(self.tile.is_empty())
        uuids = ITileDataManager(self.tile).get().get('uuids', None)

        self.assertIsNone(uuids)

        # now we 2 elements
        obj1 = self.portal['my-document']
        obj2 = self.portal['my-image']

        self.tile.populate_with_uuids([
            obj1.UID(), obj2.UID()
        ])

        # For obj2 we will assign a custom_description

        uuids = ITileDataManager(self.tile).get().get('uuids', None)
        uuids[obj2.UID()]['custom_description'] = u'New Description'

        description1 = self.tile.get_description(obj1)
        description2 = self.tile.get_description(obj2)

        # Document object should be the same as Description
        self.assertEqual(description1, obj1.Description())
        # First image should return the custom URL
        self.assertEqual(description2, u'New Description')

    def test_custom_url(self):
        # we start with an empty tile
        self.assertTrue(self.tile.is_empty())
        uuids = ITileDataManager(self.tile).get().get('uuids', None)

        self.assertIsNone(uuids)

        # now we 3 elements
        obj1 = self.portal['my-document']
        obj2 = self.portal['my-image']
        obj3 = self.portal['my-image1']

        self.tile.populate_with_uuids([
            obj1.UID(), obj2.UID(), obj3.UID()
        ])

        # For obj2 we will assign a custom_url

        uuids = ITileDataManager(self.tile).get().get('uuids', None)
        uuids[obj2.UID()]['custom_url'] = u'http://www.custom_url.com'

        url1 = self.tile.get_url(obj1)
        url2 = self.tile.get_url(obj2)
        url3 = self.tile.get_url(obj3)

        # Document object should be the same as absolute_url
        self.assertEqual(url1, obj1.absolute_url())
        # First image should return the custom URL
        self.assertEqual(url2, u'http://www.custom_url.com')
        # And second image should have the absolute_url and /view
        self.assertEqual(url3, obj3.absolute_url() + '/view')

    def test_data_converter(self):
        field = ICarouselTile['uuids']
        widget = TextLinesSortableWidget(self.request)
        conv = UUIDSFieldDataConverter(field, widget)

        value = {
            u'uuid1': {u'order': u'0'},
            u'uuid2': {u'order': u'2'},
            u'uuid3': {u'order': u'1'},
        }

        to_widget = conv.toWidgetValue(value)
        self.assertEqual(to_widget, u'uuid1\r\nuuid3\r\nuuid2')

        to_field = conv.toFieldValue(value)

        self.assertEqual(to_field, value)
        self.assertEqual(conv.toFieldValue({}), conv.field.missing_value)



# -*- coding: utf-8 -*-

from z3c.form import interfaces
from z3c.form.browser.textarea import TextAreaWidget
from z3c.form.widget import FieldWidget

import zope.component
import zope.interface
import zope.schema.interfaces


class IOverlayTextAreaWidget(interfaces.ITextAreaWidget):
    """TextArea widget for Cycle2 Overlay field."""


@zope.interface.implementer_only(IOverlayTextAreaWidget)
class OverlayTextAreaWidget(TextAreaWidget):
    """Textarea widget for Cycle2 Overlay field implementation."""

    def update(self):
        """See z3c.form.interfaces.IWidget."""
        super(TextAreaWidget, self).update()
        confvalue = self.context.get('overlay', {}).get('template')
        if confvalue is not None:
            self.value = self.context.get('overlay').get('template')


@zope.component.adapter(zope.schema.interfaces.IField, interfaces.IFormLayer)
@zope.interface.implementer(interfaces.IFieldWidget)
def OverlayTextAreaFieldWidget(field, request):
    """IFieldWidget factory for OverlayTextWidget."""
    return FieldWidget(field, OverlayTextAreaWidget(request))



# -*- coding: utf-8 -*-
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer

PROJECTNAME = 'covertile.cycle2'


@implementer(INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Do not show on Plone's list of installable profiles."""
        return [
            u'covertile.cycle2:uninstall',
        ]



# -*- coding: utf-8 -*-
from covertile.cycle2.config import PROJECTNAME
from covertile.cycle2.interfaces import IBrowserLayer
from covertile.cycle2.testing import INTEGRATION_TESTING
from plone import api
from plone.browserlayer.utils import registered_layers

import unittest


class InstallTestCase(unittest.TestCase):
    """Ensure product is properly installed."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.qi = self.portal['portal_quickinstaller']

    def test_installed(self):
        self.assertTrue(self.qi.isProductInstalled(PROJECTNAME))

    def test_dependencies_installed(self):
        self.assertTrue(self.qi.isProductInstalled('collective.js.cycle2'))

    def test_browser_layer_installed(self):
        self.assertIn(IBrowserLayer, registered_layers())

    def test_tile(self):
        tiles = api.portal.get_registry_record('plone.app.tiles')
        self.assertIn(u'covertile.cycle2.carousel', tiles)


class UninstallTestCase(unittest.TestCase):
    """Ensure product is properly removed."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.qi = self.portal['portal_quickinstaller']
        self.qi.uninstallProducts(products=[PROJECTNAME])

    def test_uninstalled(self):
        self.assertFalse(self.qi.isProductInstalled(PROJECTNAME))

    def test_browser_layer_removed(self):
        self.assertNotIn(IBrowserLayer, registered_layers())

    def test_tile_removed(self):
        tiles = api.portal.get_registry_record('plone.app.tiles')
        self.assertNotIn(u'covertile.cycle2.carousel', tiles)



# -*- coding: utf-8 -*-

from z3c.form import interfaces
from z3c.form.browser.select import SelectWidget
from z3c.form.widget import FieldWidget

import zope.component
import zope.interface
import zope.schema.interfaces


class IPagerStyleWidget(interfaces.ISelectWidget):
    """TextArea widget for Cycle2 Overlay field."""


@zope.interface.implementer_only(IPagerStyleWidget)
class PagerStyleWidget(SelectWidget):
    """Select widget implementation."""

    def update(self):
        """See z3c.form.interfaces.IWidget."""
        super(SelectWidget, self).update()
        confvalue = self.context.get('pager', {}).get('style')
        if confvalue is not None:
            self.value = self.context.get('pager').get('style')


@zope.component.adapter(zope.schema.interfaces.IChoice, interfaces.IFormLayer)
@zope.interface.implementer(interfaces.IFieldWidget)
def PagerStyleFieldWidget(field, request):
    """IFieldWidget factory for SelectWidget."""
    return FieldWidget(field, PagerStyleWidget(request))



# -*- coding: utf-8 -*-
"""Setup test fixtures.

We have to set different test fixtures depending on features we want:

plone.app.contenttypes:
    installed under Plone 4.3, if requested
"""
from collective.cover.testing import generate_jpeg
from collective.cover.tests.utils import create_standard_content_for_tests
from collective.cover.tests.utils import set_image_field
from plone.app.robotframework.testing import AUTOLOGIN_LIBRARY_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2


import os
import pkg_resources

try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    DEXTERITY_ONLY = False
else:
    # this environment variable is set in .travis.yml test matrix
    DEXTERITY_ONLY = os.environ.get('DEXTERITY_ONLY') is not None


class Fixture(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        if DEXTERITY_ONLY:
            import plone.app.contenttypes
            self.loadZCML(package=plone.app.contenttypes)
            z2.installProduct(app, 'Products.DateRecurringIndex')

        import collective.cover
        self.loadZCML(package=collective.cover)
        import covertile.cycle2
        self.loadZCML(package=covertile.cycle2)

    def setUpPloneSite(self, portal):
        if DEXTERITY_ONLY:
            self.applyProfile(portal, 'plone.app.contenttypes:default')

        self.applyProfile(portal, 'collective.cover:default')
        self.applyProfile(portal, 'collective.cover:testfixture')
        self.applyProfile(portal, 'covertile.cycle2:default')

        # setup test content
        create_standard_content_for_tests(portal)
        set_image_field(portal['my-image1'], generate_jpeg(900, 400))
        set_image_field(portal['my-image2'], generate_jpeg(900, 400))


FIXTURE = Fixture()

INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,), name='covertile.cycle2:Integration')

FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,), name='covertile.cycle2:Functional')

ROBOT_TESTING = FunctionalTesting(
    bases=(FIXTURE, AUTOLOGIN_LIBRARY_FIXTURE, z2.ZSERVER_FIXTURE),
    name='covertile.cycle2:Robot',
)



# -*- coding: utf-8 -*-
from collective.cover.interfaces import ITileEditForm
from collective.cover.tiles.configuration_view import IDefaultConfigureForm
from collective.cover.tiles.list import IListTile
from collective.cover.tiles.list import ListTile
from collective.cover.widgets.textlinessortable import TextLinesSortableFieldWidget
from covertile.cycle2 import _
from plone import api
from plone.autoform import directives as form
from plone.namedfile.field import NamedBlobImage
from plone.tiles.interfaces import ITileDataManager
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope import schema
from zope.interface import implementer
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


# Types of Pager style used in Carousel Tiles
PAGER_STYLES = SimpleVocabulary([
    SimpleTerm(value='dots', title=_(u'Dots')),
    SimpleTerm(value='numbers', title=_(u'Numbers')),
    SimpleTerm(value='thumbnails_square', title=_(u'Square Thumbnails'))
])

PAGER_TEMPLATES = {
    'dots': '<span>&bull;</span>',
    'numbers': '<strong><a href=#> {{slideNum}} </a></strong>',
    'thumbnails_square': '<a href="#"><img src="{{thumbnail}}" width=49 height=49></a>'
}

DEFAULT_PAGER_STYLE = 'dots'
DEFAULT_OVERLAY_TEMPLATE = (u'<div id="c2-overlay-title">{{title}}</div>'
                            u'<div id="c2-overlay-desc">{{desc}}</div>')


class ICarouselTile(IListTile):

    """A carousel based on the Cycle2 slideshow plugin for jQuery."""

    uuids = schema.Dict(
        title=_(u'Elements'),
        key_type=schema.TextLine(),
        value_type=schema.Dict(
            key_type=schema.TextLine(),
            value_type=schema.TextLine(),
        ),
        required=False,
    )
    form.omitted('uuids')
    form.no_omit(ITileEditForm, 'uuids')
    form.widget(uuids=TextLinesSortableFieldWidget)

    # Copied from From IListTile

    image = NamedBlobImage(
        title=_(u'Image'),
        required=False,
    )
    form.omitted('image')
    form.no_omit(IDefaultConfigureForm, 'image')

    tile_title = schema.TextLine(
        title=_(u'Tile Title'),
        required=False,
    )
    form.omitted('tile_title')
    form.no_omit(ITileEditForm, 'tile_title')

    more_link = schema.TextLine(
        title=_('Show more... link'),
        required=False,
    )
    form.omitted('more_link')
    form.no_omit(ITileEditForm, 'more_link')
    form.widget(more_link='collective.cover.tiles.edit_widgets.more_link.MoreLinkFieldWidget')

    more_link_text = schema.TextLine(
        title=_('Show more... link text'),
        required=False,
    )
    form.omitted('more_link_text')
    form.no_omit(ITileEditForm, 'more_link_text')

    form.omitted('autoplay')
    form.no_omit(ITileEditForm, 'autoplay')
    autoplay = schema.Bool(
        title=_(u'Auto play'),
        required=False,
        default=True,
    )

    # Fields specific to Carousel tiles

    pager = schema.Choice(
        title=_(u'Pager'),
        vocabulary=PAGER_STYLES,
        required=True,
        default=DEFAULT_PAGER_STYLE,
    )
    form.omitted('pager')
    form.no_omit(IDefaultConfigureForm, 'pager')
    form.widget(pager='covertile.cycle2.tiles.configuration_widgets.pagerstylewidget.PagerStyleFieldWidget')

    overlay = schema.SourceText(
        title=_(u'Overlay Template'),
        description=_(u'A Mustache-style template string, in which you can use {{title}}, {{desc}}, {{url}} or {{date}}'),
        required=False,
        default=DEFAULT_OVERLAY_TEMPLATE,
    )
    form.omitted('overlay')
    form.no_omit(IDefaultConfigureForm, 'overlay')
    form.widget(overlay='covertile.cycle2.tiles.configuration_widgets.overlaytextarea.OverlayTextAreaFieldWidget')


@implementer(ICarouselTile)
class CarouselTile(ListTile):

    """A carousel based on the Cycle2 slideshow plugin for jQuery."""

    index = ViewPageTemplateFile('templates/carousel.pt')
    is_configurable = True
    is_editable = True
    short_name = _(u'msg_short_name_carousel', default=u'C2 Carousel')
    limit = 100      # Increase hard-coded limit in ListTile

    def populate_with_object(self, obj):
        """Add an object to the carousel. This method will append new
        elements to the already existing list of items. If the object
        does not have an image associated, it will not be included and
        silently ignored.

        :param uuids: The list of objects' UUIDs to be used
        :type uuids: List of strings
        """
        if not self._has_image_field(obj):
            return
        super(CarouselTile, self).populate_with_object(obj)

    @property
    def paused(self):
        """ Return 'true' or 'false' depending on whether the carousel
            will begin in a paused state. Value is intended for Javascript."""
        autoplay = self.data.get('autoplay', None)  # autoplay init'd to None
        autoplay = True if autoplay is None else autoplay  # default to True
        paused_str = str(not autoplay).lower()
        return paused_str

    def pagerclass(self):
        """
        """
        if not self._field_is_visible('pager'):
            return ''
        else:
            tile_conf = self.get_tile_configuration()
            pager_conf = tile_conf.get('pager', None)
            pager_style = pager_conf.get('style', None)
            # stored value could be none - default should be 'dots'
            if not pager_style:
                return DEFAULT_PAGER_STYLE
            else:
                return pager_style[0]

    def pagerthumbnail(self, item):
        """Return the thumbnail of an image if the pager style is Thumbnail
        based (contains {{thumbnail}}) and the pager is visible.

        :param item: [required]
        :type item: content object
        """
        if '{{thumbnail}}' not in self.pagertemplate():
            return None  # skip expensive image processing

        if not self._field_is_visible('pager'):
            return None

        scales = item.restrictedTraverse('@@images')
        return scales.scale('image', width=49, height=49, direction='down')

    def pagertemplate(self):
        if self._field_is_visible('pager'):
            return PAGER_TEMPLATES.get(self.pagerclass(), '')
        else:
            return ''

    def overlaytemplate(self):
        if not self._field_is_visible('overlay'):
            return ''
        else:
            tile_conf = self.get_tile_configuration()
            overlay_conf = tile_conf.get('overlay', None)
            template = overlay_conf.get('template', None)
            # stored value could be none - default should be '...{{title}}...'
            if template is None:
                return DEFAULT_OVERLAY_TEMPLATE
            else:
                return template

    def get_title(self, item):
        """Get the title of the item, or the custom title if set.

        :param item: [required] The item for which we want the title
        :type item: Content object
        :returns: the item title
        :rtype: unicode
        """
        # First we get the title for the item itself
        title = item.Title()
        uuid = self.get_uuid(item)
        data_mgr = ITileDataManager(self)
        data = data_mgr.get()
        uuids = data['uuids']
        if uuid in uuids:
            if uuids[uuid].get('custom_title', u''):
                # If we had a custom title set, then get that
                title = uuids[uuid].get('custom_title')
        return title

    def get_description(self, item):
        """Get the description of the item, or the custom description
        if set.

        :param item: [required] The item for which we want the description
        :type item: Content object
        :returns: the item description
        :rtype: unicode
        """
        # First we get the url for the item itself
        description = item.Description()
        uuid = self.get_uuid(item)
        data_mgr = ITileDataManager(self)
        data = data_mgr.get()
        uuids = data['uuids']
        if uuid in uuids:
            if uuids[uuid].get('custom_description', u''):
                # If we had a custom description set, then get that
                description = uuids[uuid].get('custom_description')
        return description

    def _get_types_that_use_view_action(self):
        """Return a list of types that use the view action in listings.

        :returns: a list of content types
        :rtype: tuple
        """
        portal_properties = api.portal.get_tool('portal_properties')
        return portal_properties.site_properties.getProperty(
            'typesUseViewActionInListings', ())

    def get_url(self, item):
        """Get the URL of the item, or the custom URL if set.

        :param item: [required] The item for which we want the URL
        :type item: Content object
        :returns: the item URL
        :rtype: str
        """
        # First we get the url for the item itself
        url = item.absolute_url()
        if item.portal_type in self._get_types_that_use_view_action():
            url = url + '/view'
        uuid = self.get_uuid(item)
        data_mgr = ITileDataManager(self)
        data = data_mgr.get()
        uuids = data['uuids']
        if uuid in uuids:
            if uuids[uuid].get('custom_url', u''):
                # If we had a custom url set, then get that
                url = uuids[uuid].get('custom_url')
        return url



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

version = '1.0.0'
description = 'A carousel tile for collective.cover based on the Cycle2 slideshow plugin for jQuery.'
long_description = (
    open('README.rst').read() + '\n' +
    open('CONTRIBUTORS.rst').read() + '\n' +
    open('CHANGES.rst').read()
)

setup(
    name='covertile.cycle2',
    version=version,
    description=description,
    long_description=long_description,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: OS Independent',
        'Programming Language :: JavaScript',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='plone tile cycle2 slideshow carousel',
    author='Simples Consultoria',
    author_email='produts@simplesconsultoria.com.br',
    url='https://github.com/collective/covertile.cycle2',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['covertile'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'collective.cover',
        'collective.js.cycle2',
        'plone.api',
        'plone.autoform',
        'plone.namedfile',
        'plone.tiles',
        'Products.CMFPlone >=4.3',
        'Products.GenericSetup',
        'setuptools',
        'z3c.form',
        'zope.component',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.schema',
    ],
    extras_require={
        'test': [
            'collective.cover [test]',
            'plone.app.robotframework',
            'plone.app.testing [robot]',
            'plone.browserlayer',
            'plone.testing',
            'plone.uuid',
            'robotsuite',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
